"""
Django Improved User Library

1. define directory as module
2. set default app config

https://docs.djangoproject.com/en/stable/ref/applications/#configuring-applications

"""
# pylint: disable=invalid-name
default_app_config = 'improved_user.apps.ImprovedUserConfig'
# pylint: enable=invalid-name
